@@ -1,41 +1,10 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 Uwe Kindler
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or (at your option) any later version.
-** The licenses are as published by the Free Software Foundation
-** and appearing in the file LICENSE.LGPLv21 included in the packaging
-** of this file. Please review the following information to ensure
-** the GNU Lesser General Public License version 2.1 requirements
-** will be met: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2020 Uwe Kindler
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-2.1-or-later OR GPL-3.0-or-later
 
 #include "dockwidget.h"
 
 #include "ads_globals.h"
+#include "ads_globals_p.h"
 #include "dockareawidget.h"
 #include "dockcomponentsfactory.h"
 #include "dockcontainerwidget.h"
@@ -55,8 +24,7 @@
 #include <QTextStream>
 #include <QToolBar>
 #include <QXmlStreamWriter>
-
-static Q_LOGGING_CATEGORY(adsLog, "qtc.qmldesigner.advanceddockingsystem", QtWarningMsg)
+#include <QWindow>
 
 namespace ADS
 {
@@ -127,6 +95,7 @@ namespace ADS
         if (!m_dockArea) {
             FloatingDockContainer *floatingWidget = new FloatingDockContainer(q);
             floatingWidget->resize(q->size());
+            m_tabWidget->show();
             floatingWidget->show();
         } else {
             m_dockArea->setCurrentDockWidget(q);
@@ -202,8 +171,15 @@ namespace ADS
         d->m_tabWidget = componentsFactory()->createDockWidgetTab(this);
         d->m_toggleViewAction = new QAction(uniqueId, this);
         d->m_toggleViewAction->setCheckable(true);
-        connect(d->m_toggleViewAction, &QAction::triggered, this, &DockWidget::toggleView);
+        connect(d->m_toggleViewAction, &QAction::triggered, this, [this](bool open) {
+            // If the toggle view action mode is ActionModeShow (== m_toggleViewAction isn't
+            // checkable, see setToggleViewActionMode()), then open is always true
+            toggleView(open || !d->m_toggleViewAction->isCheckable());
+        });
         setToolbarFloatingStyle(false);
+
+        if (DockManager::testConfigFlag(DockManager::FocusHighlighting))
+            setFocusPolicy(Qt::ClickFocus);
     }
 
     DockWidget::~DockWidget()
@@ -214,15 +190,15 @@ namespace ADS
 
     void DockWidget::setToggleViewActionChecked(bool checked)
     {
-        QAction *action = d->m_toggleViewAction;
-        //action->blockSignals(true);
-        action->setChecked(checked);
-        //action->blockSignals(false);
+        d->m_toggleViewAction->setChecked(checked);
     }
 
     void DockWidget::setWidget(QWidget *widget, eInsertMode insertMode)
     {
-        QScrollArea *scrollAreaWidget = qobject_cast<QScrollArea *>(widget);
+        if (d->m_widget)
+            takeWidget();
+
+        auto scrollAreaWidget = qobject_cast<QAbstractScrollArea *>(widget);
         if (scrollAreaWidget || ForceNoScrollArea == insertMode) {
             d->m_layout->addWidget(widget);
             if (scrollAreaWidget && scrollAreaWidget->viewport())
@@ -238,11 +214,23 @@ namespace ADS
 
     QWidget *DockWidget::takeWidget()
     {
-        // TODO Shouldn't m_widget being set to nullptr?!
-        d->m_scrollArea->takeWidget();
-        d->m_layout->removeWidget(d->m_widget);
-        d->m_widget->setParent(nullptr);
-        return d->m_widget;
+        QWidget *w = nullptr;
+        if (d->m_scrollArea) {
+            d->m_layout->removeWidget(d->m_scrollArea);
+            w = d->m_scrollArea->takeWidget();
+            delete d->m_scrollArea;
+            d->m_scrollArea = nullptr;
+            d->m_widget = nullptr;
+        } else if (d->m_widget) {
+            d->m_layout->removeWidget(d->m_widget);
+            w = d->m_widget;
+            d->m_widget = nullptr;
+        }
+
+        if (w)
+            w->setParent(nullptr);
+
+        return w;
     }
 
     QWidget *DockWidget::widget() const { return d->m_widget; }
@@ -251,9 +239,9 @@ namespace ADS
 
     void DockWidget::setFeatures(DockWidgetFeatures features)
     {
-        if (d->m_features == features) {
+        if (d->m_features == features)
             return;
-        }
+
         d->m_features = features;
         emit featuresChanged(d->m_features);
         d->m_tabWidget->onDockWidgetFeaturesChanged();
@@ -274,11 +262,10 @@ namespace ADS
 
     DockContainerWidget *DockWidget::dockContainer() const
     {
-        if (d->m_dockArea) {
+        if (d->m_dockArea)
             return d->m_dockArea->dockContainer();
-        } else {
+        else
             return nullptr;
-        }
     }
 
     DockAreaWidget *DockWidget::dockAreaWidget() const { return d->m_dockArea; }
@@ -325,12 +312,6 @@ namespace ADS
 
     void DockWidget::toggleView(bool open)
     {
-        // If the toggle view action mode is ActionModeShow, then Open is always
-        // true if the sender is the toggle view action
-        QAction *action = qobject_cast<QAction *>(sender());
-        if (action == d->m_toggleViewAction && !d->m_toggleViewAction->isCheckable())
-            open = true;
-
         // If the dock widget state is different, then we really need to toggle
         // the state. If we are in the right state, then we simply make this
         // dock widget the current dock widget
@@ -347,11 +328,11 @@ namespace ADS
                                                    ? beforeDockContainerWidget->topLevelDockWidget()
                                                    : nullptr;
 
-        if (open) {
+        if (open)
             d->showDockWidget();
-        } else {
+        else
             d->hideDockWidget();
-        }
+
         d->m_closed = !open;
         //d->m_toggleViewAction->blockSignals(true);
         d->m_toggleViewAction->setChecked(open);
@@ -449,7 +430,7 @@ namespace ADS
             d->m_toggleViewAction->setToolTip(text);
 
         if (d->m_dockArea)
-            d->m_dockArea->markTitleBarMenuOutdated(); //update tabs menu
+            d->m_dockArea->markTitleBarMenuOutdated(); // update tabs menu
     }
 #endif
 
@@ -553,7 +534,17 @@ namespace ADS
 
     QSize DockWidget::minimumSizeHint() const
     {
-        if (d->m_minimumSizeHintMode == DockWidget::MinimumSizeHintFromDockWidget || !d->m_widget)
+        if (!d->m_widget)
+            return QSize(60, 40);
+
+        DockContainerWidget *container = this->dockContainer();
+        if (!container || container->isFloating()) {
+            const QSize sh = d->m_widget->minimumSizeHint();
+            const QSize s = d->m_widget->minimumSize();
+            return {std::max(s.width(), sh.width()), std::max(s.height(), sh.height())};
+        }
+
+        if (d->m_minimumSizeHintMode == DockWidget::MinimumSizeHintFromDockWidget)
             return QSize(60, 40);
         else
             return d->m_widget->minimumSizeHint();
@@ -617,4 +608,58 @@ namespace ADS
         return d->m_titleBarActions;
     }
 
+    void DockWidget::showFullScreen()
+    {
+        if (isFloating())
+            dockContainer()->floatingWidget()->showFullScreen();
+        else
+            Super::showFullScreen();
+    }
+
+    void DockWidget::showNormal()
+    {
+        if (isFloating())
+            dockContainer()->floatingWidget()->showNormal();
+        else
+            Super::showNormal();
+    }
+
+    bool DockWidget::isFullScreen() const
+    {
+        if (isFloating())
+            return dockContainer()->floatingWidget()->isFullScreen();
+        else
+            return Super::isFullScreen();
+    }
+
+    void DockWidget::setAsCurrentTab()
+    {
+        if (d->m_dockArea && !isClosed())
+            d->m_dockArea->setCurrentDockWidget(this);
+    }
+
+    bool DockWidget::isTabbed() const
+    {
+        return d->m_dockArea && (d->m_dockArea->openDockWidgetsCount() > 1);
+    }
+
+    bool DockWidget::isCurrentTab() const
+    {
+        return d->m_dockArea && (d->m_dockArea->currentDockWidget() == this);
+    }
+
+    void DockWidget::raise()
+    {
+        if (isClosed())
+            return;
+
+        setAsCurrentTab();
+        if (isInFloatingContainer())
+        {
+            auto floatingWindow = window();
+            floatingWindow->raise();
+            floatingWindow->activateWindow();
+        }
+    }
+
 } // namespace ADS
