@@ -1,45 +1,30 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "contextpanewidgetimage.h"
 
-#include "ui_contextpanewidgetimage.h"
-#include "ui_contextpanewidgetborderimage.h"
+#include "filewidget.h"
+#include "qmleditorwidgetstr.h"
+
 #include <qmljs/qmljspropertyreader.h>
+#include <utils/layoutbuilder.h>
+
+#include <QApplication>
+#include <QBoxLayout>
+#include <QButtonGroup>
+#include <QDebug>
+#include <QDir>
 #include <QFile>
 #include <QFileInfo>
-#include <QDir>
-#include <QPixmap>
-#include <QPainter>
 #include <QGraphicsEffect>
 #include <QMouseEvent>
+#include <QPainter>
+#include <QPixmap>
+#include <QRadioButton>
 #include <QScrollArea>
 #include <QSlider>
 #include <QStyle>
 #include <QToolButton>
-#include <QDebug>
 
 namespace QmlEditorWidgets {
 
@@ -66,35 +51,76 @@ bool WheelFilter::eventFilter(QObject *obj, QEvent *event)
     return QObject::eventFilter(obj, event);
 }
 
-ContextPaneWidgetImage::ContextPaneWidgetImage(QWidget *parent, bool borderImage) :
-    QWidget(parent),
-    ui(nullptr), uiBorderImage(nullptr), previewWasVisible(false)
+ContextPaneWidgetImage::ContextPaneWidgetImage(QWidget *parent, bool borderImage)
+    : QWidget(parent)
+    , m_isBorderImage(borderImage)
 {
     LabelFilter *labelFilter = new LabelFilter(this);
 
-    m_borderImage = borderImage;
+    const auto radioButton = [] (const QString &icon, const QString &toolTip) {
+        auto result = new QRadioButton;
+        result->setIcon(QIcon(":/qmldesigner/images/" + icon + ".png"));
+        result->setIconSize({24, 24});
+        result->setToolTip(toolTip);
+        return result;
+    };
 
-    if (m_borderImage) {
-        uiBorderImage = new Ui::ContextPaneWidgetBorderImage;
-        uiBorderImage->setupUi(this);
-        m_fileWidget = uiBorderImage->fileWidget;
-        m_sizeLabel = uiBorderImage->sizeLabel;
-        uiBorderImage->label->setToolTip(tr("Double click for preview."));
-        uiBorderImage->label->installEventFilter(labelFilter);
+    m_previewLabel = new QLabel;
+    m_previewLabel->setToolTip(Tr::tr("Double click for preview."));
+    m_previewLabel->setFixedSize(76, 76);
+    m_previewLabel->installEventFilter(labelFilter);
 
+    m_sizeLabel = new QLabel;
+    m_sizeLabel->setAlignment(Qt::AlignHCenter);
+    m_fileWidget = new FileWidget;
 
-        connect(uiBorderImage->verticalTileRadioButton, &QRadioButton::toggled,
+    if (m_isBorderImage) {
+        m_borderImage.horizontalTileRadioButton = radioButton("tile-icon-hor-crop",
+            Tr::tr("Repeat vertically. Tiles the image until there is no more space. May crop the last image."));
+        m_borderImage.horizontalStretchRadioButton = radioButton("horizontal-scale-icon",
+            Tr::tr("Stretch vertically. Scales the image to fit to the available area."));
+        m_borderImage.horizontalTileRadioButtonNoCrop = radioButton("tile-icon-hor-scale",
+            Tr::tr("Round. Like Repeat, but scales the images down to ensure that the last image is not cropped."));
+        auto hRadioButtons = new QButtonGroup(this);
+        hRadioButtons->addButton(m_borderImage.horizontalTileRadioButton);
+        hRadioButtons->addButton(m_borderImage.horizontalStretchRadioButton);
+        hRadioButtons->addButton(m_borderImage.horizontalTileRadioButtonNoCrop);
+
+        m_borderImage.verticalTileRadioButton = radioButton("tile-icon-vert-crop",
+            Tr::tr("Repeat horizontally. Tiles the image until there is no more space. May crop the last image."));
+        m_borderImage.verticalStretchRadioButton = radioButton("vertical-scale-icon",
+            Tr::tr("Stretch horizontally. Scales the image to fit to the available area."));
+        m_borderImage.verticalTileRadioButtonNoCrop = radioButton("tile-icon-vert-scale",
+            Tr::tr("Round. Like Repeat, but scales the images down to ensure that the last image is not cropped."));
+        auto vRadioButtons = new QButtonGroup(this);
+        vRadioButtons->addButton(m_borderImage.verticalTileRadioButton);
+        vRadioButtons->addButton(m_borderImage.verticalStretchRadioButton);
+        vRadioButtons->addButton(m_borderImage.verticalTileRadioButtonNoCrop);
+
+        using namespace Layouting;
+        Row {
+            Column { m_previewLabel, m_sizeLabel, },
+            Column {
+                m_fileWidget,
+                Grid {
+                    m_borderImage.horizontalTileRadioButton, m_borderImage.horizontalStretchRadioButton, m_borderImage.horizontalTileRadioButtonNoCrop, br,
+                    m_borderImage.verticalTileRadioButton, m_borderImage.verticalStretchRadioButton, m_borderImage.verticalTileRadioButtonNoCrop,
+                },
+            },
+        }.attachTo(this);
+
+        connect(m_borderImage.verticalTileRadioButton, &QRadioButton::toggled,
                 this, &ContextPaneWidgetImage::onVerticalStretchChanged);
-        connect(uiBorderImage->verticalStretchRadioButton, &QRadioButton::toggled,
+        connect(m_borderImage.verticalStretchRadioButton, &QRadioButton::toggled,
                 this, &ContextPaneWidgetImage::onVerticalStretchChanged);
-        connect(uiBorderImage->verticalTileRadioButtonNoCrop, &QRadioButton::toggled,
+        connect(m_borderImage.verticalTileRadioButtonNoCrop, &QRadioButton::toggled,
                 this, &ContextPaneWidgetImage::onVerticalStretchChanged);
 
-        connect(uiBorderImage->horizontalTileRadioButton, &QRadioButton::toggled,
+        connect(m_borderImage.horizontalTileRadioButton, &QRadioButton::toggled,
                 this, &ContextPaneWidgetImage::onHorizontalStretchChanged);
-        connect(uiBorderImage->horizontalStretchRadioButton, &QRadioButton::toggled,
+        connect(m_borderImage.horizontalStretchRadioButton, &QRadioButton::toggled,
                 this, &ContextPaneWidgetImage::onHorizontalStretchChanged);
-        connect(uiBorderImage->horizontalTileRadioButtonNoCrop, &QRadioButton::toggled,
+        connect(m_borderImage.horizontalTileRadioButtonNoCrop, &QRadioButton::toggled,
                 this, &ContextPaneWidgetImage::onHorizontalStretchChanged);
         PreviewLabel *previewLabel = previewDialog()->previewLabel();
         connect(previewLabel, &PreviewLabel::leftMarginChanged,
@@ -105,26 +131,44 @@ ContextPaneWidgetImage::ContextPaneWidgetImage(QWidget *parent, bool borderImage
                 this, &ContextPaneWidgetImage::onTopMarginsChanged);
         connect(previewLabel, &PreviewLabel::bottomMarginChanged,
                 this, &ContextPaneWidgetImage::onBottomMarginsChanged);
-
     } else {
-        ui = new Ui::ContextPaneWidgetImage;
-        ui->setupUi(this);
-        ui->label->setToolTip(tr("Double click for preview."));
-        ui->label->installEventFilter(labelFilter);
-        m_fileWidget = ui->fileWidget;
-        m_sizeLabel = ui->sizeLabel;
+        m_image.stretchRadioButton = radioButton("scale-icon",
+            Tr::tr("The image is scaled to fit."));
+        m_image.horizontalStretchRadioButton = radioButton("horizontal-scale-icon",
+            Tr::tr("The image is stretched horizontally and tiled vertically."));
+        m_image.verticalStretchRadioButton = radioButton("vertical-scale-icon",
+            Tr::tr("The image is stretched vertically and tiled horizontally."));
 
-        connect(ui->stretchRadioButton, &QRadioButton::toggled,
+        m_image.tileRadioButton = radioButton("tile-icon",
+            Tr::tr("The image is duplicated horizontally and vertically."));
+        m_image.preserveAspectFitRadioButton = radioButton("aspect-fit-icon",
+            Tr::tr("The image is scaled uniformly to fit without cropping."));
+        m_image.cropAspectFitRadioButton = radioButton("aspect-crop-icon",
+            Tr::tr("The image is scaled uniformly to fill, cropping if necessary."));
+
+        using namespace Layouting;
+        Row {
+            Column { m_previewLabel, m_sizeLabel, },
+            Column {
+                m_fileWidget,
+                Grid {
+                    m_image.stretchRadioButton, m_image.horizontalStretchRadioButton, m_image.verticalStretchRadioButton, br,
+                    m_image.tileRadioButton, m_image.preserveAspectFitRadioButton, m_image.cropAspectFitRadioButton,
+                },
+            },
+        }.attachTo(this);
+
+        connect(m_image.stretchRadioButton, &QRadioButton::toggled,
                 this, &ContextPaneWidgetImage::onStretchChanged);
-        connect(ui->tileRadioButton, &QRadioButton::toggled,
+        connect(m_image.tileRadioButton, &QRadioButton::toggled,
                 this, &ContextPaneWidgetImage::onStretchChanged);
-        connect(ui->horizontalStretchRadioButton, &QRadioButton::toggled,
+        connect(m_image.horizontalStretchRadioButton, &QRadioButton::toggled,
                 this, &ContextPaneWidgetImage::onStretchChanged);
-        connect(ui->verticalStretchRadioButton, &QRadioButton::toggled,
+        connect(m_image.verticalStretchRadioButton, &QRadioButton::toggled,
                 this, &ContextPaneWidgetImage::onStretchChanged);
-        connect(ui->preserveAspectFitRadioButton, &QRadioButton::toggled,
+        connect(m_image.preserveAspectFitRadioButton, &QRadioButton::toggled,
                 this, &ContextPaneWidgetImage::onStretchChanged);
-        connect(ui->cropAspectFitRadioButton, &QRadioButton::toggled,
+        connect(m_image.cropAspectFitRadioButton, &QRadioButton::toggled,
                 this, &ContextPaneWidgetImage::onStretchChanged);
     }
     previewDialog();
@@ -138,17 +182,9 @@ ContextPaneWidgetImage::ContextPaneWidgetImage(QWidget *parent, bool borderImage
 
 }
 
-ContextPaneWidgetImage::~ContextPaneWidgetImage()
-{
-    if (ui)
-        delete ui;
-    if (uiBorderImage)
-        delete uiBorderImage;
-}
-
 void ContextPaneWidgetImage::setProperties(QmlJS::PropertyReader *propertyReader)
 {
-    if (m_borderImage) {
+    if (m_isBorderImage) {
 
         int leftBorder = 0;
         int rightBorder = 0;
@@ -170,34 +206,34 @@ void ContextPaneWidgetImage::setProperties(QmlJS::PropertyReader *propertyReader
             if (fillMode.contains(QLatin1String("BorderImage.")))
                 fillMode.remove(QLatin1String("BorderImage."));
 
-            uiBorderImage->horizontalStretchRadioButton->setChecked(true);
+            m_borderImage.horizontalStretchRadioButton->setChecked(true);
 
             if (fillMode == QLatin1String("Stretch"))
-                uiBorderImage->horizontalStretchRadioButton->setChecked(true);
+                m_borderImage.horizontalStretchRadioButton->setChecked(true);
             if (fillMode == QLatin1String("Repeat"))
-                uiBorderImage->horizontalTileRadioButton->setChecked(true);
+                m_borderImage.horizontalTileRadioButton->setChecked(true);
             if (fillMode == QLatin1String("Round"))
-                uiBorderImage->horizontalTileRadioButtonNoCrop->setChecked(true);
+                m_borderImage.horizontalTileRadioButtonNoCrop->setChecked(true);
         } else {
             //uiBorderImage
-            uiBorderImage->horizontalStretchRadioButton->setChecked(true);
+            m_borderImage.horizontalStretchRadioButton->setChecked(true);
         }
         if (propertyReader->hasProperty(QLatin1String("verticalTileMode"))) {
             QString fillMode = propertyReader->readProperty(QLatin1String("verticalTileMode")).toString();
             if (fillMode.contains(QLatin1String("BorderImage.")))
                 fillMode.remove(QLatin1String("BorderImage."));
 
-            uiBorderImage->verticalStretchRadioButton->setChecked(true);
+            m_borderImage.verticalStretchRadioButton->setChecked(true);
 
             if (fillMode == QLatin1String("Stretch"))
-                uiBorderImage->verticalStretchRadioButton->setChecked(true);
+                m_borderImage.verticalStretchRadioButton->setChecked(true);
             if (fillMode == QLatin1String("Repeat"))
-                uiBorderImage->verticalTileRadioButton->setChecked(true);
+                m_borderImage.verticalTileRadioButton->setChecked(true);
             if (fillMode == QLatin1String("Round"))
-                uiBorderImage->verticalTileRadioButtonNoCrop->setChecked(true);
+                m_borderImage.verticalTileRadioButtonNoCrop->setChecked(true);
         } else {
             //uiBorderImage
-            uiBorderImage->verticalStretchRadioButton->setChecked(true);
+            m_borderImage.verticalStretchRadioButton->setChecked(true);
         }
     } else {
         if (propertyReader->hasProperty(QLatin1String("fillMode"))) {
@@ -205,20 +241,20 @@ void ContextPaneWidgetImage::setProperties(QmlJS::PropertyReader *propertyReader
             if (fillMode.contains(QLatin1String("Image.")))
                 fillMode.remove(QLatin1String("Image."));
 
-            ui->stretchRadioButton->setChecked(true);
+            m_image.stretchRadioButton->setChecked(true);
 
             if (fillMode == QLatin1String("Image.Tile") || fillMode == QLatin1String("Tile"))
-                ui->tileRadioButton->setChecked(true);
+                m_image.tileRadioButton->setChecked(true);
             if (fillMode == QLatin1String("Image.TileVertically") || fillMode == QLatin1String("TileVertically"))
-                ui->horizontalStretchRadioButton->setChecked(true);
+                m_image.horizontalStretchRadioButton->setChecked(true);
             if (fillMode == QLatin1String("Image.TileHorizontally") || fillMode == QLatin1String("TileHorizontally"))
-                ui->verticalStretchRadioButton->setChecked(true);
+                m_image.verticalStretchRadioButton->setChecked(true);
             if (fillMode == QLatin1String("Image.PreserveAspectFit") || fillMode == QLatin1String("PreserveAspectFit"))
-                ui->preserveAspectFitRadioButton->setChecked(true);
+                m_image.preserveAspectFitRadioButton->setChecked(true);
             if (fillMode == QLatin1String("Image.PreserveAspectCrop") || fillMode == QLatin1String("PreserveAspectCrop"))
-                ui->cropAspectFitRadioButton->setChecked(true);
+                m_image.cropAspectFitRadioButton->setChecked(true);
         } else {
-            ui->stretchRadioButton->setChecked(true);
+            m_image.stretchRadioButton->setChecked(true);
         }
     }
     if (propertyReader->hasProperty(QLatin1String("source"))) {
@@ -277,7 +313,7 @@ void PreviewDialog::setZoom(int z)
 
 void PreviewDialog::setIsBorderImage(bool b)
 {
-    m_borderImage = b;
+    m_isBorderImage = b;
     m_label->setIsBorderImage(b);
 }
 
@@ -289,17 +325,17 @@ PreviewLabel *PreviewDialog::previewLabel() const
 void ContextPaneWidgetImage::onStretchChanged()
 {
     QString stretch;
-    if (ui->stretchRadioButton->isChecked())
+    if (m_image.stretchRadioButton->isChecked())
         stretch = QLatin1String("Image.Stretch");
-    else if (ui->tileRadioButton->isChecked())
+    else if (m_image.tileRadioButton->isChecked())
         stretch = QLatin1String("Image.Tile");
-    else if (ui->horizontalStretchRadioButton->isChecked())
+    else if (m_image.horizontalStretchRadioButton->isChecked())
         stretch = QLatin1String("Image.TileVertically");
-    else if (ui->verticalStretchRadioButton->isChecked())
+    else if (m_image.verticalStretchRadioButton->isChecked())
         stretch = QLatin1String("Image.TileHorizontally");
-    else if (ui->preserveAspectFitRadioButton->isChecked())
+    else if (m_image.preserveAspectFitRadioButton->isChecked())
         stretch = QLatin1String("Image.PreserveAspectFit");
-    else if (ui->cropAspectFitRadioButton->isChecked())
+    else if (m_image.cropAspectFitRadioButton->isChecked())
         stretch = QLatin1String("Image.PreserveAspectCrop");
 
     if (stretch == QLatin1String("Image.Stretch"))
@@ -311,11 +347,11 @@ void ContextPaneWidgetImage::onStretchChanged()
 void ContextPaneWidgetImage::onHorizontalStretchChanged()
 {
     QString stretch;
-    if (uiBorderImage->horizontalStretchRadioButton->isChecked())
+    if (m_borderImage.horizontalStretchRadioButton->isChecked())
         stretch = QLatin1String("BorderImage.Stretch");
-    if (uiBorderImage->horizontalTileRadioButton->isChecked())
+    if (m_borderImage.horizontalTileRadioButton->isChecked())
         stretch = QLatin1String("BorderImage.Repeat");
-    if (uiBorderImage->horizontalTileRadioButtonNoCrop->isChecked())
+    if (m_borderImage.horizontalTileRadioButtonNoCrop->isChecked())
         stretch = QLatin1String("BorderImage.Round");
 
     if (stretch == QLatin1String("BorderImage.Stretch"))
@@ -327,11 +363,11 @@ void ContextPaneWidgetImage::onHorizontalStretchChanged()
 void ContextPaneWidgetImage::onVerticalStretchChanged()
 {
     QString stretch;
-    if (uiBorderImage->verticalStretchRadioButton->isChecked())
+    if (m_borderImage.verticalStretchRadioButton->isChecked())
         stretch = QLatin1String("BorderImage.Stretch");
-    if (uiBorderImage->verticalTileRadioButton->isChecked())
+    if (m_borderImage.verticalTileRadioButton->isChecked())
         stretch = QLatin1String("BorderImage.Repeat");
-    if (uiBorderImage->verticalTileRadioButtonNoCrop->isChecked())
+    if (m_borderImage.verticalTileRadioButtonNoCrop->isChecked())
         stretch = QLatin1String("BorderImage.Round");
 
     if (stretch == QLatin1String("BorderImage.Stretch"))
@@ -353,9 +389,9 @@ void ContextPaneWidgetImage::onPixmapDoubleClicked()
 {
     previewDialog()->setParent(parentWidget()->parentWidget());
     previewDialog()->setMaximumSize(previewDialog()->parentWidget()->size() - QSize(150, 100));
-    if (m_borderImage)
+    if (m_isBorderImage)
         previewDialog()->setZoom(4);
-    previewDialog()->setIsBorderImage(m_borderImage);
+    previewDialog()->setIsBorderImage(m_isBorderImage);
 
     QPoint p = parentWidget()->pos();
     p = p + QPoint(-2, -2);
@@ -481,7 +517,7 @@ void ContextPaneWidgetImage::setPixmap(const QString &fileName)
     QPixmap pix(76,76);
     pix.fill(Qt::black);
 
-    if (m_borderImage) {
+    if (m_isBorderImage) {
         QString localFileName(fileName);
         if (QFileInfo::exists(fileName)) {
             if (fileName.endsWith(QLatin1String("sci"))) {
@@ -497,7 +533,7 @@ void ContextPaneWidgetImage::setPixmap(const QString &fileName)
                     localFileName = QFileInfo(fileName).absoluteDir().absolutePath() + QLatin1Char('/') + pixmapFileName;
                     previewDialog()->previewLabel()->setMargins(left, top, right, bottom);
                 } else { // sci file not parsed correctly
-                    uiBorderImage->sizeLabel->clear();
+                    m_sizeLabel->clear();
                     return;
                 }
             }
@@ -505,18 +541,18 @@ void ContextPaneWidgetImage::setPixmap(const QString &fileName)
             if (source.isNull())
                 source = pix;
             previewDialog()->setPixmap(source, previewDialog()->zoom());
-            uiBorderImage->sizeLabel->setText(QString::number(source.width()) + QLatin1Char('x')
-                                              + QString::number(source.height()));
+            m_sizeLabel->setText(QString::number(source.width()) + QLatin1Char('x')
+                                 + QString::number(source.height()));
             QPainter p(&pix);
             Qt::TileRule horizontalTileMode = Qt::StretchTile;
             Qt::TileRule verticalTileMode = Qt::StretchTile;
-            if (uiBorderImage->horizontalTileRadioButton->isChecked())
+            if (m_borderImage.horizontalTileRadioButton->isChecked())
                 horizontalTileMode =Qt::RepeatTile;
-            if (uiBorderImage->horizontalTileRadioButtonNoCrop->isChecked())
+            if (m_borderImage.horizontalTileRadioButtonNoCrop->isChecked())
                 horizontalTileMode =Qt::RoundTile;
-            if (uiBorderImage->verticalTileRadioButton->isChecked())
+            if (m_borderImage.verticalTileRadioButton->isChecked())
                 verticalTileMode =Qt::RepeatTile;
-            if (uiBorderImage->verticalTileRadioButtonNoCrop->isChecked())
+            if (m_borderImage.verticalTileRadioButtonNoCrop->isChecked())
                 verticalTileMode =Qt::RoundTile;
             QTileRules rules(horizontalTileMode, verticalTileMode);
             PreviewLabel *previewLabel = previewDialog()->previewLabel();
@@ -524,46 +560,46 @@ void ContextPaneWidgetImage::setPixmap(const QString &fileName)
             qDrawBorderPixmap(&p, QRect(0, 0, 76, 76), margins, source, source.rect(), margins, rules);
             //p.drawPixmap(0,0,76,76, source);
         } else {
-            uiBorderImage->sizeLabel->clear();
+            m_sizeLabel->clear();
         }
-        uiBorderImage->label->setPixmap(pix);
+        m_previewLabel->setPixmap(pix);
     } else {
         if (QFileInfo::exists(fileName)) {
             QPixmap source(fileName);
             previewDialog()->setPixmap(source, 1);
-            ui->sizeLabel->setText(QString::number(source.width()) + QLatin1Char('x') + QString::number(source.height()));
+            m_sizeLabel->setText(QString::number(source.width()) + QLatin1Char('x') + QString::number(source.height()));
             QPainter p(&pix);
-            if (ui->stretchRadioButton->isChecked()) {
+            if (m_image.stretchRadioButton->isChecked()) {
                 p.drawPixmap(0,0,76,76, source);
-            } else if (ui->tileRadioButton->isChecked()) {
+            } else if (m_image.tileRadioButton->isChecked()) {
                 QPixmap small = source.scaled(38,38);
                 p.drawTiledPixmap(0,0,76,76, small);
-            } else if (ui->horizontalStretchRadioButton->isChecked()) {
+            } else if (m_image.horizontalStretchRadioButton->isChecked()) {
                 QPixmap small = source.scaled(38,38);
                 QPixmap half = pix.scaled(38, 76);
                 QPainter p2(&half);
                 p2.drawTiledPixmap(0,0,38,76, small);
                 p.drawPixmap(0,0,76,76, half);
-            } else if (ui->verticalStretchRadioButton->isChecked()) {
+            } else if (m_image.verticalStretchRadioButton->isChecked()) {
                 QPixmap small = source.scaled(38,38);
                 QPixmap half = pix.scaled(76, 38);
                 QPainter p2(&half);
                 p2.drawTiledPixmap(0,0,76,38, small);
                 p.drawPixmap(0,0,76,76, half);
-            } else if (ui->preserveAspectFitRadioButton->isChecked()) {
+            } else if (m_image.preserveAspectFitRadioButton->isChecked()) {
                 QPixmap preserved = source.scaledToWidth(76);
                 int offset = (76 - preserved.height()) / 2;
                 p.drawPixmap(0, offset, 76, preserved.height(), source);
-            } else if (ui->cropAspectFitRadioButton->isChecked()) {
+            } else if (m_image.cropAspectFitRadioButton->isChecked()) {
                 QPixmap cropped = source.scaledToHeight(76);
                 int offset = (76 - cropped.width()) / 2;
                 p.drawPixmap(offset, 0, cropped.width(), 76, source);
             }
         } else {
-            ui->sizeLabel->clear();
+            m_sizeLabel->clear();
         }
 
-        ui->label->setPixmap(pix);
+        m_previewLabel->setPixmap(pix);
 
     }
 }
@@ -578,32 +614,16 @@ PreviewDialog* ContextPaneWidgetImage::previewDialog()
     return m_previewDialog.data();
 }
 
-void ContextPaneWidgetImage::changeEvent(QEvent *e)
-{
-    QWidget::changeEvent(e);
-    switch (e->type()) {
-    case QEvent::LanguageChange:
-        if (ui)
-            ui->retranslateUi(this);
-        if (uiBorderImage)
-            uiBorderImage->retranslateUi(this);
-        break;
-    default:
-        break;
-    }
-}
-
-
 void ContextPaneWidgetImage::hideEvent(QHideEvent * event)
 {
-    previewWasVisible = previewDialog()->isVisible();
+    m_previewWasVisible = previewDialog()->isVisible();
     previewDialog()->hide();
     QWidget::hideEvent(event);
 }
 
 void ContextPaneWidgetImage::showEvent(QShowEvent* event)
 {
-    if (previewWasVisible)
+    if (m_previewWasVisible)
         previewDialog()->show();
     QWidget::showEvent(event);
 }
@@ -642,7 +662,7 @@ void PreviewLabel::setZoom(int z)
 
 void PreviewLabel::setIsBorderImage(bool b)
 {
-    m_borderImage = b;
+    m_isBorderImage = b;
 }
 
 void PreviewLabel::setMargins(int left, int top, int right, int bottom)
@@ -656,7 +676,7 @@ void PreviewLabel::setMargins(int left, int top, int right, int bottom)
 void PreviewLabel::paintEvent(QPaintEvent *event)
 {
     QLabel::paintEvent(event);
-    if (m_borderImage) {
+    if (m_isBorderImage) {
 
         QPainter p(this);
 
@@ -704,7 +724,7 @@ static inline bool rangeCheck(int target, int pos)
 
 void PreviewLabel::mousePressEvent(QMouseEvent * event)
 {
-    if (!m_borderImage) {
+    if (!m_isBorderImage) {
         QLabel::mouseMoveEvent(event);
         return;
     }
@@ -754,7 +774,7 @@ void PreviewLabel::mousePressEvent(QMouseEvent * event)
 
 void PreviewLabel::mouseReleaseEvent(QMouseEvent * event)
 {
-    if (!m_borderImage)
+    if (!m_isBorderImage)
         return QLabel::mouseMoveEvent(event);
 
     if (m_dragging_left || m_dragging_top || m_dragging_right|| m_dragging_bottom) {
@@ -793,7 +813,7 @@ static inline int limitPositive(int i)
 
 void PreviewLabel::mouseMoveEvent(QMouseEvent * event)
 {
-    if (!m_borderImage) {
+    if (!m_isBorderImage) {
         QLabel::mouseMoveEvent(event);
         return;
     }
@@ -871,7 +891,7 @@ void PreviewLabel::leaveEvent(QEvent* event )
 
 PreviewDialog::PreviewDialog(QWidget *parent) : DragWidget(parent)
 {
-    m_borderImage = false;
+    m_isBorderImage = false;
     setAutoFillBackground(true);
 
     m_label = new PreviewLabel(this);
@@ -884,7 +904,6 @@ PreviewDialog::PreviewDialog(QWidget *parent) : DragWidget(parent)
     QVBoxLayout *layout = new QVBoxLayout(this);
     QHBoxLayout *horizontalLayout = new QHBoxLayout();
     QHBoxLayout *horizontalLayout2 = new QHBoxLayout();
-    layout->setMargin(0);
     layout->setContentsMargins(2, 2, 2, 16);
     layout->setSpacing(4);
     QToolButton *toolButton = new QToolButton(this);
@@ -919,7 +938,9 @@ PreviewDialog::PreviewDialog(QWidget *parent) : DragWidget(parent)
 
     connect(m_slider, &QSlider::valueChanged, this, &PreviewDialog::onSliderMoved);
 
-    foreach (QWidget *childWidget, findChildren<QWidget*>()) {
+
+    const QList<QWidget *> children = findChildren<QWidget*>();
+    for (QWidget *childWidget : children) {
         childWidget->installEventFilter(wheelFilter);
     }
 }
@@ -934,7 +955,7 @@ void PreviewDialog::setPixmap(const QPixmap &p, int zoom)
     m_label->adjustSize();
     m_zoom = zoom;
     m_label->setZoom(m_zoom);
-    QSize size = m_label->pixmap()->size() + QSize(54, 44);
+    QSize size = m_label->pixmap().size() + QSize(54, 44);
     if (size.width() < 180)
         size.setWidth(180);
     resize(size);
@@ -942,7 +963,7 @@ void PreviewDialog::setPixmap(const QPixmap &p, int zoom)
 
 void PreviewDialog::wheelEvent(QWheelEvent* event)
 {
-    int delta = event->delta();
+    const auto delta = event->angleDelta().y();
     event->accept();
     if (delta >  0) {
         if (m_zoom == 1)
