@@ -1,44 +1,20 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+
+#include "debuggerruncontrol.h"
 
 #include "debuggermainwindow.h"
-#include "debuggerruncontrol.h"
+#include "debuggertr.h"
 #include "terminal.h"
 
-#include "analyzer/analyzermanager.h"
 #include "console/console.h"
 #include "debuggeractions.h"
-#include "debuggercore.h"
 #include "debuggerengine.h"
 #include "debuggerinternalconstants.h"
 #include "debuggerkitinformation.h"
-#include "debuggerplugin.h"
 #include "debuggerrunconfigurationaspect.h"
 #include "breakhandler.h"
 #include "enginemanager.h"
-#include "shared/peutils.h"
 
 #include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/devicesupport/deviceprocessesdialog.h>
@@ -47,18 +23,20 @@
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/projectexplorericons.h>
+#include <projectexplorer/projectmanager.h>
 #include <projectexplorer/runconfigurationaspects.h>
-#include <projectexplorer/session.h>
+#include <projectexplorer/projectmanager.h>
 #include <projectexplorer/target.h>
 #include <projectexplorer/taskhub.h>
 #include <projectexplorer/toolchain.h>
 
 #include <utils/algorithm.h>
 #include <utils/checkablemessagebox.h>
+#include <utils/environment.h>
 #include <utils/fileutils.h>
 #include <utils/portlist.h>
+#include <utils/process.h>
 #include <utils/qtcassert.h>
-#include <utils/qtcprocess.h>
 #include <utils/temporarydirectory.h>
 #include <utils/temporaryfile.h>
 #include <utils/url.h>
@@ -92,15 +70,16 @@ DebuggerEngine *createPdbEngine();
 DebuggerEngine *createQmlEngine();
 DebuggerEngine *createLldbEngine();
 DebuggerEngine *createUvscEngine();
+DebuggerEngine *createDapEngine();
 
 static QString noEngineMessage()
 {
-   return DebuggerPlugin::tr("Unable to create a debugging engine.");
+   return Tr::tr("Unable to create a debugging engine.");
 }
 
 static QString noDebuggerInKitMessage()
 {
-   return DebuggerPlugin::tr("The kit does not have a debugger set.");
+   return Tr::tr("The kit does not have a debugger set.");
 }
 
 class CoreUnpacker final : public RunWorker
@@ -130,7 +109,7 @@ private:
         }
 
         m_coreUnpackProcess.setWorkingDirectory(TemporaryDirectory::masterDirectoryFilePath());
-        connect(&m_coreUnpackProcess, &QtcProcess::done, this, [this] {
+        connect(&m_coreUnpackProcess, &Process::done, this, [this] {
             if (m_coreUnpackProcess.error() == QProcess::UnknownError) {
                 reportStopped();
                 return;
@@ -138,7 +117,7 @@ private:
             reportFailure("Error unpacking " + m_coreFilePath.toUserOutput());
         });
 
-        const QString msg = DebuggerRunTool::tr("Unpacking core file to %1");
+        const QString msg = Tr::tr("Unpacking core file to %1");
         appendMessage(msg.arg(m_tempCoreFilePath.toUserOutput()), LogMessageFormat);
 
         if (m_coreFilePath.endsWith(".lzo")) {
@@ -153,8 +132,8 @@ private:
             appendMessage(msg.arg(m_tempCoreFilePath.toUserOutput()), LogMessageFormat);
             m_tempCoreFile.setFileName(m_tempCoreFilePath.path());
             m_tempCoreFile.open(QFile::WriteOnly);
-            connect(&m_coreUnpackProcess, &QtcProcess::readyReadStandardOutput, this, [this] {
-                m_tempCoreFile.write(m_coreUnpackProcess.readAllStandardOutput());
+            connect(&m_coreUnpackProcess, &Process::readyReadStandardOutput, this, [this] {
+                m_tempCoreFile.write(m_coreUnpackProcess.readAllRawStandardOutput());
             });
             m_coreUnpackProcess.setCommand({"gzip", {"-c", "-d", m_coreFilePath.path()}});
             reportStarted();
@@ -169,7 +148,7 @@ private:
     QFile m_tempCoreFile;
     FilePath m_coreFilePath;
     FilePath m_tempCoreFilePath;
-    QtcProcess m_coreUnpackProcess;
+    Process m_coreUnpackProcess;
 };
 
 class DebuggerRunToolPrivate
@@ -205,13 +184,13 @@ void DebuggerRunTool::setStartMode(DebuggerStartMode startMode)
 
         // FIXME: This is horribly wrong.
         // get files from all the projects in the session
-        QList<Project *> projects = SessionManager::projects();
-        if (Project *startupProject = SessionManager::startupProject()) {
+        QList<Project *> projects = ProjectManager::projects();
+        if (Project *startupProject = ProjectManager::startupProject()) {
             // startup project first
             projects.removeOne(startupProject);
             projects.insert(0, startupProject);
         }
-        for (Project *project : qAsConst(projects))
+        for (Project *project : std::as_const(projects))
             m_runParameters.projectSourceFiles.append(project->files(Project::SourceFiles));
         if (!projects.isEmpty())
             m_runParameters.projectSourceDirectory = projects.first()->projectDirectory();
@@ -244,7 +223,7 @@ void DebuggerRunTool::setSysRoot(const Utils::FilePath &sysRoot)
 void DebuggerRunTool::setSymbolFile(const FilePath &symbolFile)
 {
     if (symbolFile.isEmpty())
-        reportFailure(tr("Cannot debug: Local executable is not set."));
+        reportFailure(Tr::tr("Cannot debug: Local executable is not set."));
     m_runParameters.symbolFile = symbolFile;
 }
 
@@ -469,17 +448,18 @@ void DebuggerRunTool::start()
     if (m_runParameters.startMode == StartInternal
             && m_runParameters.inferior.command.isEmpty()
             && m_runParameters.interpreter.isEmpty()) {
-        reportFailure(tr("No executable specified."));
+        reportFailure(Tr::tr("No executable specified."));
         return;
     }
 
     // QML and/or mixed are not prepared for it.
-    setSupportsReRunning(!m_runParameters.isQmlDebugging);
+//    setSupportsReRunning(!m_runParameters.isQmlDebugging);
+    setSupportsReRunning(false); // FIXME: Broken in general.
 
     // FIXME: Disabled due to Android. Make Android device report available ports instead.
 //    int portsUsed = portsUsedByDebugger();
 //    if (portsUsed > device()->freePorts().count()) {
-//        reportFailure(tr("Cannot debug: Not enough free ports available."));
+//        reportFailure(Tr::tr("Cannot debug: Not enough free ports available."));
 //        return;
 //    }
 
@@ -493,7 +473,7 @@ void DebuggerRunTool::start()
             && Utils::is64BitWindowsBinary(m_runParameters.inferior.command.executable())
             && !Utils::is64BitWindowsBinary(m_runParameters.debugger.command.executable())) {
         reportFailure(
-            DebuggerPlugin::tr(
+            Tr::tr(
                 "%1 is a 64 bit executable which can not be debugged by a 32 bit Debugger.\n"
                 "Please select a 64 bit Debugger in the kit settings for this kit.")
                 .arg(m_runParameters.inferior.command.executable().toUserOutput()));
@@ -501,7 +481,7 @@ void DebuggerRunTool::start()
     }
 
     Utils::globalMacroExpander()->registerFileVariables(
-                "DebuggedExecutable", tr("Debugged executable"),
+                "DebuggedExecutable", Tr::tr("Debugged executable"),
                 [this] { return m_runParameters.inferior.command.executable(); }
     );
 
@@ -515,7 +495,7 @@ void DebuggerRunTool::start()
                 break;
             case CdbEngineType:
                 if (!HostOsInfo::isWindowsHost()) {
-                    reportFailure(tr("Unsupported CDB host system."));
+                    reportFailure(Tr::tr("Unsupported CDB host system."));
                     return;
                 }
                 m_engine = createCdbEngine();
@@ -530,10 +510,13 @@ void DebuggerRunTool::start()
             case UvscEngineType:
                 m_engine = createUvscEngine();
                 break;
+            case DapEngineType:
+                m_engine = createDapEngine();
+                break;
             default:
                 if (!m_runParameters.isQmlDebugging) {
                     reportFailure(noEngineMessage() + '\n' +
-                        DebuggerPlugin::tr("Specify Debugger settings in Projects > Run."));
+                        Tr::tr("Specify Debugger settings in Projects > Run."));
                     return;
                 }
                 // Can happen for pure Qml.
@@ -578,7 +561,7 @@ void DebuggerRunTool::start()
     connect(m_engine, &DebuggerEngine::attachToCoreRequested, this, [this](const QString &coreFile) {
         auto rc = new RunControl(ProjectExplorer::Constants::DEBUG_RUN_MODE);
         rc->copyDataFromRunControl(runControl());
-        auto name = QString(tr("%1 - Snapshot %2").arg(runControl()->displayName()).arg(++d->snapshotCounter));
+        auto name = QString(Tr::tr("%1 - Snapshot %2").arg(runControl()->displayName()).arg(++d->snapshotCounter));
         auto debugger = new DebuggerRunTool(rc);
         debugger->setStartMode(AttachToCore);
         debugger->setRunControlName(name);
@@ -606,7 +589,7 @@ void DebuggerRunTool::start()
         ++d->engineStopsNeeded;
     }
 
-    if (m_runParameters.startMode == StartInternal) {
+    if (m_runParameters.startMode != AttachToCore) {
         QStringList unhandledIds;
         bool hasQmlBreakpoints = false;
         for (const GlobalBreakpoint &gbp : BreakpointManager::globalBreakpoints()) {
@@ -620,37 +603,35 @@ void DebuggerRunTool::start()
             }
         }
         if (!unhandledIds.isEmpty()) {
-            QString warningMessage =
-                    DebuggerPlugin::tr("Some breakpoints cannot be handled by the debugger "
-                                       "languages currently active, and will be ignored.<p>"
-                                       "Affected are breakpoints %1")
-                    .arg(unhandledIds.join(", "));
+            QString warningMessage = Tr::tr("Some breakpoints cannot be handled by the debugger "
+                                            "languages currently active, and will be ignored.<p>"
+                                            "Affected are breakpoints %1")
+                                         .arg(unhandledIds.join(", "));
 
             if (hasQmlBreakpoints) {
-                warningMessage += "<p>" +
-                    DebuggerPlugin::tr("QML debugging needs to be enabled both in the Build "
-                                       "and the Run settings.");
+                warningMessage += "<p>"
+                                  + Tr::tr("QML debugging needs to be enabled both in the Build "
+                                           "and the Run settings.");
             }
 
             showMessage(warningMessage, LogWarning);
 
-            static bool checked = true;
-            if (checked)
-                CheckableMessageBox::information(Core::ICore::dialogParent(),
-                                                 tr("Debugger"),
-                                                 warningMessage,
-                                                 tr("&Show this message again."),
-                                                 &checked, QDialogButtonBox::Ok);
+            static bool doNotShowAgain = false;
+            CheckableMessageBox::information(Core::ICore::dialogParent(),
+                                             Tr::tr("Debugger"),
+                                             warningMessage,
+                                             &doNotShowAgain,
+                                             QMessageBox::Ok);
         }
     }
 
-    appendMessage(tr("Debugging %1 ...").arg(m_runParameters.inferior.command.toUserOutput()),
+    appendMessage(Tr::tr("Debugging %1 ...").arg(m_runParameters.inferior.command.toUserOutput()),
                   NormalMessageFormat);
     QString debuggerName = m_engine->objectName();
     if (m_engine2)
         debuggerName += ' ' + m_engine2->objectName();
 
-    const QString message = tr("Starting debugger \"%1\" for ABI \"%2\"...")
+    const QString message = Tr::tr("Starting debugger \"%1\" for ABI \"%2\"...")
             .arg(debuggerName).arg(m_runParameters.toolChainAbi.toString());
     DebuggerMainWindow::showStatusMessage(message, 10000);
 
@@ -691,9 +672,9 @@ void DebuggerRunTool::handleEngineFinished(DebuggerEngine *engine)
     if (--d->engineStopsNeeded == 0) {
         QString cmd = m_runParameters.inferior.command.toUserOutput();
         QString msg = engine->runParameters().exitCode // Main engine.
-            ? tr("Debugging of %1 has finished with exit code %2.")
+            ? Tr::tr("Debugging of %1 has finished with exit code %2.")
                 .arg(cmd).arg(engine->runParameters().exitCode.value())
-            : tr("Debugging of %1 has finished.").arg(cmd);
+            : Tr::tr("Debugging of %1 has finished.").arg(cmd);
         appendMessage(msg, NormalMessageFormat);
         reportStopped();
     }
@@ -728,7 +709,7 @@ DebugServerPortsGatherer *DebuggerRunTool::portsGatherer() const
     return d->portsGatherer;
 }
 
-void DebuggerRunTool::setSolibSearchPath(const QStringList &list)
+void DebuggerRunTool::setSolibSearchPath(const Utils::FilePaths &list)
 {
     m_runParameters.solibSearchPath = list;
 }
@@ -756,7 +737,7 @@ bool DebuggerRunTool::fixupParameters()
             if (rp.qmlServer.port() <= 0) {
                 rp.qmlServer = Utils::urlFromLocalHostAndFreePort();
                 if (rp.qmlServer.port() <= 0) {
-                    reportFailure(DebuggerPlugin::tr("Not enough free ports for QML debugging."));
+                    reportFailure(Tr::tr("Not enough free ports for QML debugging."));
                     return false;
                 }
             }
@@ -768,7 +749,7 @@ bool DebuggerRunTool::fixupParameters()
         }
     }
 
-    if (!debuggerSettings()->autoEnrichParameters.value()) {
+    if (debuggerSettings()->autoEnrichParameters.value()) {
         const FilePath sysroot = rp.sysRoot;
         if (rp.debugInfoLocation.isEmpty())
             rp.debugInfoLocation = sysroot / "/usr/lib/debug";
@@ -808,16 +789,16 @@ bool DebuggerRunTool::fixupParameters()
     }
 
     if (HostOsInfo::isWindowsHost()) {
+        // Otherwise command lines with '> tmp.log' hang.
         ProcessArgs::SplitError perr;
-        rp.inferior.command.setArguments(
-                ProcessArgs::prepareArgs(rp.inferior.command.arguments(), &perr,
-                                         HostOsInfo::hostOs(), nullptr,
-                                         &rp.inferior.workingDirectory).toWindowsArgs());
+        ProcessArgs::prepareArgs(rp.inferior.command.arguments(), &perr,
+                                 HostOsInfo::hostOs(), nullptr,
+                                 &rp.inferior.workingDirectory).toWindowsArgs();
         if (perr != ProcessArgs::SplitOk) {
             // perr == BadQuoting is never returned on Windows
             // FIXME? QTCREATORBUG-2809
-            reportFailure(DebuggerPlugin::tr("Debugging complex command lines "
-                                             "is currently not supported on Windows."));
+            reportFailure(Tr::tr("Debugging complex command lines "
+                                 "is currently not supported on Windows."));
             return false;
         }
     }
@@ -857,8 +838,8 @@ DebuggerRunTool::DebuggerRunTool(RunControl *runControl, AllowTerminal allowTerm
     runControl->setIcon(ProjectExplorer::Icons::DEBUG_START_SMALL_TOOLBAR);
     runControl->setPromptToStop([](bool *optionalPrompt) {
         return RunControl::showPromptToStopDialog(
-            DebuggerRunTool::tr("Close Debugging Session"),
-            DebuggerRunTool::tr("A debugging session is still in progress. "
+            Tr::tr("Close Debugging Session"),
+            Tr::tr("A debugging session is still in progress. "
                                 "Terminating the session in the current"
                                 " state can leave the target in an inconsistent state."
                                 " Would you still like to terminate it?"),
@@ -882,8 +863,10 @@ DebuggerRunTool::DebuggerRunTool(RunControl *runControl, AllowTerminal allowTerm
     m_runParameters.debugger = DebuggerKitAspect::runnable(kit);
     m_runParameters.cppEngineType = DebuggerKitAspect::engineType(kit);
 
-    if (QtSupport::QtVersion *qtVersion = QtSupport::QtKitAspect::qtVersion(kit))
+    if (QtSupport::QtVersion *qtVersion = QtSupport::QtKitAspect::qtVersion(kit)) {
         m_runParameters.qtPackageSourceLocation = qtVersion->qtPackageSourcePath().toString();
+        m_runParameters.qtSourceLocation = qtVersion->sourcePath();
+    }
 
     if (auto aspect = runControl->aspect<DebuggerRunConfigurationAspect>()) {
         if (!aspect->useCppDebugger)
@@ -907,27 +890,30 @@ DebuggerRunTool::DebuggerRunTool(RunControl *runControl, AllowTerminal allowTerm
 
     Runnable inferior = runControl->runnable();
     const FilePath &debuggerExecutable = m_runParameters.debugger.command.executable();
-    inferior.command.setExecutable(inferior.command.executable().onDevice(debuggerExecutable));
-    inferior.workingDirectory = inferior.workingDirectory.onDevice(debuggerExecutable);
+    inferior.command.setExecutable(debuggerExecutable.withNewMappedPath(inferior.command.executable()));
+    inferior.workingDirectory = debuggerExecutable.withNewMappedPath(inferior.workingDirectory);
     // Normalize to work around QTBUG-17529 (QtDeclarative fails with 'File name case mismatch'...)
     inferior.workingDirectory = inferior.workingDirectory.normalizedPathName();
     m_runParameters.inferior = inferior;
 
     setUseTerminal(allowTerminal == DoAllowTerminal && m_runParameters.useTerminal);
 
-    const QByteArray envBinary = qgetenv("QTC_DEBUGGER_PATH");
+    const QString envBinary = qtcEnvironmentVariable("QTC_DEBUGGER_PATH");
     if (!envBinary.isEmpty())
-        m_runParameters.debugger.command.setExecutable(FilePath::fromString(QString::fromLocal8Bit(envBinary)));
+        m_runParameters.debugger.command.setExecutable(FilePath::fromString(envBinary));
 
     if (Project *project = runControl->project()) {
         m_runParameters.projectSourceDirectory = project->projectDirectory();
         m_runParameters.projectSourceFiles = project->files(Project::SourceFiles);
+    } else {
+        m_runParameters.projectSourceDirectory = m_runParameters.debugger.command.executable().parentDir();
+        m_runParameters.projectSourceFiles.clear();
     }
 
     m_runParameters.toolChainAbi = ToolChainKitAspect::targetAbi(kit);
 
     bool ok = false;
-    const int nativeMixedOverride = qEnvironmentVariableIntValue("QTC_DEBUGGER_NATIVE_MIXED", &ok);
+    const int nativeMixedOverride = qtcEnvironmentVariableIntValue("QTC_DEBUGGER_NATIVE_MIXED", &ok);
     if (ok)
         m_runParameters.nativeMixedEnabled = bool(nativeMixedOverride);
 
@@ -946,12 +932,11 @@ DebuggerRunTool::DebuggerRunTool(RunControl *runControl, AllowTerminal allowTerm
         }
     }
 
-    m_runParameters.dumperPath = Core::ICore::resourcePath("debugger/");
     if (QtSupport::QtVersion *baseQtVersion = QtSupport::QtKitAspect::qtVersion(kit)) {
-        QtSupport::QtVersionNumber qtVersion = baseQtVersion->qtVersion();
-        m_runParameters.fallbackQtVersion = 0x10000 * int(qtVersion.majorVersion)
-                                            + 0x100 * int(qtVersion.minorVersion)
-                                            + int(qtVersion.patchVersion);
+        const QVersionNumber qtVersion = baseQtVersion->qtVersion();
+        m_runParameters.fallbackQtVersion = 0x10000 * qtVersion.majorVersion()
+                                            + 0x100 * qtVersion.minorVersion()
+                                            + qtVersion.microVersion();
     }
 }
 
@@ -964,7 +949,7 @@ void DebuggerRunTool::addSolibSearchDir(const QString &str)
 {
     QString path = str;
     path.replace("%{sysroot}", m_runParameters.sysRoot.toString());
-    m_runParameters.solibSearchPath.append(path);
+    m_runParameters.solibSearchPath.append(FilePath::fromString(path));
 }
 
 DebuggerRunTool::~DebuggerRunTool()
@@ -1058,14 +1043,37 @@ DebugServerRunner::DebugServerRunner(RunControl *runControl, DebugServerPortsGat
             cmd.setExecutable(commandLine().executable()); // FIXME: Case should not happen?
         } else {
             cmd.setExecutable(runControl->device()->debugServerPath());
-            if (cmd.isEmpty())
-                cmd.setExecutable(runControl->device()->filePath("gdbserver"));
+
+            if (cmd.isEmpty()) {
+                if (runControl->device()->osType() == Utils::OsTypeMac) {
+                    const FilePath debugServerLocation = runControl->device()->filePath(
+                        "/Applications/Xcode.app/Contents/SharedFrameworks/LLDB.framework/"
+                        "Resources/debugserver");
+
+                    if (debugServerLocation.isExecutableFile()) {
+                        cmd.setExecutable(debugServerLocation);
+                    } else {
+                        // TODO: In the future it is expected that the debugserver will be
+                        // replaced by lldb-server. Remove the check for debug server at that point.
+                        const FilePath lldbserver
+                            = runControl->device()->filePath("lldb-server").searchInPath();
+                        if (lldbserver.isExecutableFile())
+                            cmd.setExecutable(lldbserver);
+                    }
+                } else {
+                    cmd.setExecutable(runControl->device()->filePath("gdbserver"));
+                }
+            }
             args.clear();
-            if (cmd.executable().toString().contains("lldb-server")) {
+            if (cmd.executable().baseName().contains("lldb-server")) {
                 args.append("platform");
                 args.append("--listen");
                 args.append(QString("*:%1").arg(portsGatherer->gdbServer().port()));
                 args.append("--server");
+            } else if (cmd.executable().baseName() == "debugserver") {
+                args.append(QString("*:%1").arg(portsGatherer->gdbServer().port()));
+                args.append("--attach");
+                args.append(QString::number(m_pid.pid()));
             } else {
                 // Something resembling gdbserver
                 if (m_useMulti)
@@ -1095,4 +1103,14 @@ void DebugServerRunner::setAttachPid(ProcessHandle pid)
     m_pid = pid;
 }
 
-} // namespace Debugger
+// DebuggerRunWorkerFactory
+
+DebuggerRunWorkerFactory::DebuggerRunWorkerFactory()
+{
+    setProduct<DebuggerRunTool>();
+    addSupportedRunMode(ProjectExplorer::Constants::DEBUG_RUN_MODE);
+    addSupportedDeviceType(ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE);
+    addSupportedDeviceType("DockerDeviceType");
+}
+
+} // Debugger
